const bmbextPrefObserver =
{
	_branch: null,
	_registered: false,
	handleEvent: function BMBPO_handleEvent(aEvent)
	{
		switch(aEvent.type)
		{
			case "load":
				this.register("BMBExt.");
				break;
			
			case "unload":
				this.unregister();
				break;
		}
	},
	register: function BMBPO_register(aPrefRoot)
	{
		if (this._registered)
		{
			dump("bmbextPrefObserver: can't register observer. observer already registered!\n");
			return;
		}
		
		dump("bmbext.PrefObserver: adding observer ...\n");
		var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		
		this._branch = prefService.getBranch(aPrefRoot);
		this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this._branch.addObserver("", this, false);
		this._registered = true;
	},
	
	unregister: function BMBPO_unregister()
	{
		if (!this._registered)
		{
			dump("bmbextPrefObserver: can't unregister observer. observer not registered!\n");
			return;
		}
		
		dump("bmbext.PrefObserver: removing observer ...\n");
		this._branch.removeObserver("", this);
		this._registered = false;
	},
	
	QueryInterface: function BMBPO_QueryInterface(aIID)
	{
		if (!aIID.equals(Components.interfaces.nsISupports) && !aIID.equals(Components.interfaces.nsIObserver))
		{
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	
	observe: function BMBPO_observe(aSubject, aTopic, aData)
	{
		dump("bmbext.PrefObserver: observe() called ...\n");
		if (aTopic != "nsPref:changed")
		{
			dump("bmbext.PrefObserver: " + aTopic + " is not the right topic for me.\n");
			return;
		}
		
		// aSubject is the nsIPrefBranch we're observing (after appropriate QI)
		// aData is the name of the pref that's been changed (relative to aSubject)
		//dump("subject (pref-branch): " + aSubject.root + "\n");
		var component = 0;
		if (aData.search(/^Book\./) == 0)
		{
			component = 1;
		}
		else if (aData.search(/^History\./) == 0)
		{
			component = 2;
		}
		else if (aData.search(/^Sort\./) == 0)
		{
			component = 3;
		}
		
		switch(component)
		{
			case 1:
				this.handleBMB(aData);
				break;
			
			case 2:
				this.handleHMB(aData);
				break;
			
			case 3:
				this.handleSort(aData);
				break;
			
			case 0:
			default:
				bmbext.loadGPrefs();
				bmbextHistory.loadGPrefs();
				break;
		}
	},
	
	handleBMB: function BMBPO_handleBMB(aData)
	{
		if (typeof(bmbext) == "undefined")
		{
			dump("bmbext.PrefObserver: bmbext is undefined!\n");
			return;
		}
		dump("bmbext.PrefObserver: name of the pref which was changed: " + aData +"\n");
		bmbext.loadPrefs();
		
		switch(aData)
		{
			case "Book.bookmarkmenu.hide":
			case "Book.settings.MiddleClickOpens":
			case "Book.settings.RightClickOpens":
				//bmbext.loadPrefs();
				bmbext.showHide();
				break;
			
			case "Book.dropmarker.hide":
				//bmbext.loadPrefs();
				bmbext.showHideDropmarker();
				break;
			
			/*case "Book.settings.hide":
				bmbext.loadPrefs();
				break;*/
			
			case "Book.settings.ShowElements":
				bmbext.showHideOptElements();
				break;
			
			default:
				break;
		}
	},
	
	handleHMB: function BMBPO_handleHMB(aData)
	{
		if (typeof(bmbextHistory) == "undefined")
		{
			dump("bmbext.PrefObserver: bmbextHistory is undefined!\n");
			return;
		}
		dump("bmbext.PrefObserver: name of the pref which was changed: " + aData +"\n");
		bmbextHistory.loadPrefs();
		
		switch(aData)
		{
			case "History.historymenu.hide":
				bmbextHistory.hideHistoryMenu();
				break;
			
			case "History.dropmarker.hide":
				bmbextHistory.showHideDropmarker();
				break;
			
			default:
				break;
		}
	},
	
	handleSort: function BMBPO_handleSort(aData)
	{
		if (typeof(bmbextSort) == "undefined")
		{
			dump("bmbext.PrefObserver: bmbextSort is undefined!\n");
			return;
		}
		dump("bmbext.PrefObserver: name of the pref which was changed: " + aData +"\n");
		bmbextSort.loadPrefs();
	}
}


window.addEventListener("load", bmbextPrefObserver, false);
window.addEventListener("unload", bmbextPrefObserver, false);