/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookmarks Menu Button.
 *
 * The Initial Developer of the Original Code is Ronny Perinke.
 * Portions created by the Initial Developer are Copyright (C) 2004-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *   Ronny Perinke <www.sephiroth-j.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
const bmbextPrefs = {
	_PrefsIFace: null,
	prefRoot: function(aIndex)
	{
		switch(aIndex)
		{
			case 0:
			default:
				return "BMBExt.";
			
			case 1:
				return "BMBExt.Book.";
			
			case 2:
				return "BMBExt.History.";
				
			case 3:
				return "BMBExt.Sort.";
		}
	},
	get StringBundle()
	{
		var stringBundle = document.getElementById("bmbextstr");
		delete this.StringBundle;
		this.StringBundle = stringBundle;
		return stringBundle;
	},
	get PrefsIFace()
	{
		if (!this._PrefsIFace)
		{
			this.PrefsIFace = this.prefRoot;
			this.__defineGetter__("PrefsIFace", function() { return this._PrefsIFace; });
		}
		return this._PrefsIFace;
	},
	set PrefsIFace(aBranch)
	{
		if (typeof(aBranch) != "string")
		{
			throw Components.results.NS_ERROR_ILLEGAL_VALUE + "\nValue must be a string type";
		}
		
		this._PrefsIFace = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
		this._PrefsIFace = this._PrefsIFace.getBranch((aBranch != "")?aBranch:this.prefRoot);
	},
	StringToInt: function(aString)
	{
		if (aString == null || aString == "")
		{
			//return -1;
			return "NaN";
		}
		else
		{
			return parseInt(aString);
		}
	},
	
	selectTab: function()
	{
		if (window.arguments)
		{
			var TabIndex = window.arguments[0];
			if (TabIndex > 0)
			{
				document.getElementById("tabbox").selectedIndex = TabIndex;
			}
		}
	},
	
	onLoad: function()
	{
		var dialog = document.getElementById("bmb-ext-prefs");
		if (dialog.getAttribute("width") != "")
		{
			dialog.removeAttribute("width");
		}
		if (dialog.getAttribute("height") != "")
		{
			dialog.removeAttribute("height");
		}
		this.selectTab();
		this.loadPrefs();
		sizeToContent();
	},
	
	loadPrefs: function()
	{
		//var alertText;
		var loadError = {raised: false, message: ""};
		
		// BOOKMARK PREFS
		this.PrefsIFace = this.prefRoot(1);
		try
		{
			document.getElementById("bmbext-hide-BM").checked = this.PrefsIFace.getBoolPref("bookmarkmenu.hide");
			
			document.getElementById("bmbext-hide-BMBS").checked = this.PrefsIFace.getBoolPref("settings.hide");
			
			document.getElementById("bmbext-hide-dropmarker").checked = this.PrefsIFace.getBoolPref("dropmarker.hide");
			
			document.getElementById("bmbext-MiddleClickOpens").selectedIndex = this.PrefsIFace.getIntPref("settings.MiddleClickOpens");
			document.getElementById("bmbext-RightClickOpens").selectedIndex = this.PrefsIFace.getIntPref("settings.RightClickOpens");
			
			let selectedElements = this.PrefsIFace.getIntPref("settings.ShowElements");
			let listbox = document.getElementById("bmbext-bmb-ShowElements");
			for(let i = 0, item, value; i < listbox.itemCount; i++)
			{
				item = listbox.getItemAtIndex(i);
				value = this.StringToInt(item.value);
				if ((selectedElements&value) === value)
				{
					item.checked = true;
				}
			}
		}
		catch (e)
		{
			loadError.raised = true;
			loadError.message += e + "\n";
		}
		
		// HISTORY PREFS
		this.PrefsIFace = this.prefRoot(2);
		try
		{
			document.getElementById("bmbext-history-hide-GM").checked = this.PrefsIFace.getBoolPref("historymenu.hide");
			
			document.getElementById("bmbext-history-hide-BMBS").checked = this.PrefsIFace.getBoolPref("settings.hide");
			
			document.getElementById("bmbext-history-hide-dropmarker").checked = this.PrefsIFace.getBoolPref("dropmarker.hide");
			
			document.getElementById("bmbext-history-MiddleClickOpens").selectedIndex = this.PrefsIFace.getIntPref("settings.MiddleClickOpens");
			document.getElementById("bmbext-history-RightClickOpens").selectedIndex = this.PrefsIFace.getIntPref("settings.RightClickOpens");
			document.getElementById("bmbext-history-GroupMethod").selectedIndex = this.PrefsIFace.getIntPref("settings.GroupMethod");
			if (!("RESULTS_AS_DATE_DOMAIN_SITE_QUERY" in Components.interfaces.nsINavHistoryQueryOptions))
			{
				document.getElementById("bmbext-history-GroupMethod").firstChild.lastChild.disabled = true;
			}
			document.getElementById("bmbext-history-MaxRecentHistory").value = String(this.PrefsIFace.getIntPref("settings.MaxRecentHistory"));
		}
		catch (e)
		{
			loadError.raised = true;
			loadError.message += e + "\n";
		}
		
		// SORT PREFS
		this.PrefsIFace =  this.prefRoot(3);
		try
		{
			document.getElementById("bmbext-sort-showsortmenu").checked = this.PrefsIFace.getBoolPref("settings.showextrasortmenu");
			document.getElementById("bmbext-sort-recursive").checked = this.PrefsIFace.getBoolPref("settings.sortrecursive");
			document.getElementById("bmbext-sort-exclude-toolbarfolder").checked = this.PrefsIFace.getBoolPref("settings.excludetoolbar");
			document.getElementById("bmbext-sort-after").selectedIndex = this.PrefsIFace.getIntPref("settings.sortcriterion") - 1;
			document.getElementById("bmbext-sort-alphabetic").selectedIndex = (this.PrefsIFace.getIntPref("settings.alphabetic") <0)?1:0;
			document.getElementById("bmbext-sort-folders").selectedIndex = this.PrefsIFace.getIntPref("settings.sortfolderstyle") - 1;
		}
		catch (e)
		{
			loadError.raised = true;
			loadError.message += e + "\n";
		}
		if (loadError.raised)
		{
			var alertText = this.StringBundle.getString("bmb-ext.settings.loaderr");
			alert(alertText + loadError.message);
		}
	},
	
	onAccept: function()
	{
		var alertText;
		var value;
		var saveError = {raised: false, message: ""};
		
		// BOOKMARK PREFS
		this.PrefsIFace = this.prefRoot(1);
		try
		{
			let value = "";
			
			this.PrefsIFace.setBoolPref("bookmarkmenu.hide", document.getElementById("bmbext-hide-BM").checked);
			
			this.PrefsIFace.setBoolPref("settings.hide", document.getElementById("bmbext-hide-BMBS").checked);
			
			this.PrefsIFace.setBoolPref("dropmarker.hide", document.getElementById("bmbext-hide-dropmarker").checked);
			
			value = document.getElementById("bmbext-MiddleClickOpens").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.MiddleClickOpens", value);
			
			value = document.getElementById("bmbext-RightClickOpens").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.RightClickOpens", value);
			
			let listbox = document.getElementById("bmbext-bmb-ShowElements");
			value = 0;
			for(let i = 0, item; i < listbox.itemCount; i++)
			{
				item = listbox.getItemAtIndex(i);
				if (item.checked)
				{
					value |= this.StringToInt(item.value);
				}
			}
			this.PrefsIFace.setIntPref("settings.ShowElements", value);
		}
		catch (e)
		{
			saveError.raised = true;
			saveError.message += e + "\n";
		}
		
		// HISTORY PREFS
		this.PrefsIFace = this.prefRoot(2);
		try
		{
			let value = "";
			
			this.PrefsIFace.setBoolPref("historymenu.hide", document.getElementById("bmbext-history-hide-GM").checked);
			
			this.PrefsIFace.setBoolPref("settings.hide", document.getElementById("bmbext-history-hide-BMBS").checked);
			
			this.PrefsIFace.setBoolPref("dropmarker.hide", document.getElementById("bmbext-history-hide-dropmarker").checked);
			
			value = document.getElementById("bmbext-history-MiddleClickOpens").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.MiddleClickOpens", value);
			
			value = document.getElementById("bmbext-history-RightClickOpens").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.RightClickOpens", value);
			
			value = document.getElementById("bmbext-history-GroupMethod").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.GroupMethod", value);
			
			value = document.getElementById("bmbext-history-MaxRecentHistory").value;
			value = this.StringToInt(value);
			if (value <= 0)
			{
				value = 0;
			}
			else if (value > 20)
			{
				var check = confirm(this.StringBundle.getFormattedString("bmb-ext.settings.MaxRecentHistoryHigh", [value]));
				if (!check)
				{
					return false;
				}
			}
			this.PrefsIFace.setIntPref("settings.MaxRecentHistory", value);
		}
		catch (e)
		{
			saveError.raised = true;
			saveError.message += e + "\n";
		}
		
		// SORT PREFS
		this.PrefsIFace = this.prefRoot(3);
		try
		{
			let value = "";
			
			this.PrefsIFace.setBoolPref("settings.showextrasortmenu", document.getElementById("bmbext-sort-showsortmenu").checked);
			this.PrefsIFace.setBoolPref("settings.sortrecursive", document.getElementById("bmbext-sort-recursive").checked);
			this.PrefsIFace.setBoolPref("settings.excludetoolbar", document.getElementById("bmbext-sort-exclude-toolbarfolder").checked);
			
			value = document.getElementById("bmbext-sort-after").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.sortcriterion", value);
			
			value = document.getElementById("bmbext-sort-alphabetic").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.alphabetic", value);
			
			value = document.getElementById("bmbext-sort-folders").value;
			value = this.StringToInt(value);
			this.PrefsIFace.setIntPref("settings.sortfolderstyle", value);
		}
		catch (e)
		{
			saveError.raised = true;
			saveError.message += e + "\n";
		}
		
		if (saveError.raised)
		{
			alertText = this.StringBundle.getString("bmb-ext.settings.saveerr");
			alert(alertText + saveError.message);
			return false;
		}
		
		return true;
	}
}